#!/usr/bin/env Rscript

# Packages -----
library(tidyverse)
library(reshape2)
library(caret)
library(pROC)
library(gbm)
library(e1071)
library(glmnet)
library(raster)
library(rgdal)
library(gridExtra)
library(doMC)
library(nnet)
library(DHARMa)
library(SuppDists)
registerDoMC(cores = 4)

# Clean up -----
rm(list = ls())
# change cutoff for where we denote if a cell has experienced a change in crop extent
cutoff <- .025

# getting list of GBM file names
# NB: ignore "GBM"—relates to previous versions/projects where GBMs were fitted.
file.list <- list.files(path = "Data/ModellingData")
file.list <- list.files(path = ".")

### Prep: creating lists used to assign model outputs
# creating list used to assign values to model output if cells experienced a change in cropland extent
# Change these based on specific file names—-ours have "GBM" in the title
region_list <- gsub("GBM.Data.22.Feb.","",file.list)
region_list <- gsub(".csv","",region_list)
region_list <- as.character(region_list)

region_list_model_change <- paste(gsub(" ","",region_list),"model_change",sep = "_") 
# list used to assign values to model output for model that estimates the change in cropland extent by cell
region_list_model_extentchange_increase <- paste(gsub(" ","",region_list),"model_extentchange_increase",sep = "_")
region_list_model_extentchange_decrease <- paste(gsub(" ","",region_list),"model_extentchange_decrease",sep = "_") 

# Data prep: Set predictor names ----
# I am selecting the columns by name not index to account for possible changes
# This is currently only used to test for scaled predictors
predictor_vars_crop <- c("dist_50k", # This is travel time to nearest city. We did not update variable name from earlier analyses.
                         "ag_suitability",
                         "PA_binary",
                         "prop_adj_to_predict_from_crop", 
                         "prop_to_predict_from_crop", 
                         "change_demand_crop",
                         "prop_to_predict_from_cell_delta_crop",
                         "prop_to_predict_from_pasture")
# And plot variables to test residuals
plot_vars_crop <- c('dist_50k_log' ,
                           'dist_50k_log_sq' ,
                           'ag_suitability' ,
                           'PA_binary',
                           'prop_adj_to_predict_from_crop' ,
                           'prop_adj_crop_sq' ,
                           'change_demand_crop' ,
                           'prop_to_predict_from_crop' ,
                           'prop_crop_sq' ,
                           'prop_to_predict_from_pasture' ,
                           'prop_pasture_sq' ,
                           'prop_to_predict_from_cell_delta_crop' ,
                           'prop_cell_delta_crop_sq')

# creating lists to savemodels
change_list_crop <- list() # first model step
proportion_list_crop_increase <- list() # second model step
proportion_list_crop_decrease <- list() # second model step

# Creating list of names used to save files
file.names.save <- paste(region_list,
                         "Cutoff_025_Crop.rda",
                         sep = "")
mods.prob <- list()
mods.increase <- list()
mods.decrease <- list()

coef.prob <- list()
coef.increase <- list()
coef.decrease <- list()

t1 = Sys.time()
### big loop to cycle through each file in file.list
for (k in 1:length(region_list)) {
  print(region_list[k])
  
  # importing files
  dat_orig <- read.csv(file.list[k],
                       header = TRUE,
                       stringsAsFactors = FALSE)
  # All non PA areas have value of 0
  dat_orig$PA.Binary[-which(dat_orig$PA.Binary == 1)] <- 0
  
  # creating columns for for change in crop between 2002 and 2007 and 2007 and 2012
  dat_orig$crop_extent_change_2007.2002 <- dat_orig$Prop.Crop.2007 - dat_orig$Prop.Crop.2002
  dat_orig$crop_extent_change_2012.2007 <- dat_orig$Prop.Crop.2012 - dat_orig$Prop.Crop.2007
  # creating column for has_changed_crop between 2007 and 2012
  dat_orig$has_changed_crop_2012.2007 <- NA
  dat_orig$has_changed_crop_2012.2007[dat_orig$crop_extent_change_2012.2007 >= cutoff] <- "Increase"
  dat_orig$has_changed_crop_2012.2007[dat_orig$crop_extent_change_2012.2007 <= -cutoff] <- "Decrease"
  dat_orig$has_changed_crop_2012.2007[abs(dat_orig$crop_extent_change_2012.2007) < cutoff] <- "aNoChange"
  # Data prep: Remove unnecessary columns, rename predictor_vars ----
  # I am renaming a) for consistency b) so that we can more easily understand
  dat_orig <- dplyr::select(dat_orig,
                            row_index = Row.Index,
                            ISO_numeric = ISO.Numeric,
                            IUCN_region = IUCN.Region,
                            dist_50k = Dist.50k,
                            ag_suitability = Ag.suitability,
                            PA_binary = PA.Binary,
                            change_demand_crop = CountryIncrease.Crop.2002.2007,
                            prop_adj_to_predict_from_crop = Prop.AdjacentCrop.2007,
                            prop_to_predict_from_crop = Prop.Crop.2007,
                            prop_to_predict_from_cell_delta_crop = crop_extent_change_2007.2002,
                            crop_extent_change_2012.2007 = crop_extent_change_2012.2007,
                            prop_to_predict_from_pasture = Prop.Pasture.2007,
                            prop_crop_cell_2002 = Prop.Crop.2002,
                            prop_crop_cell_2012 = Prop.Crop.2012,
                            prop_change_cell_crop_2007.2012 = crop_extent_change_2012.2007,
                            has_changed_crop = has_changed_crop_2012.2007,
                            GDD_10c = GDD_10c)
  # Data prep: Getting model data
  # Limiting to specific variables and complete cases
  dat_mod <- dat_orig[,c(predictor_vars_crop, "has_changed_crop", "prop_change_cell_crop_2007.2012","ISO_numeric")]
  dat_mod <- dat_mod[complete.cases(dat_mod),]

  # Removing original dat
  rm(dat_orig)
  
  # Adding columns for square transformations
  dat_mod$prop_crop_sq <- dat_mod$prop_to_predict_from_crop^2
  dat_mod$prop_pasture_sq <- dat_mod$prop_to_predict_from_pasture^2
  dat_mod$prop_cell_delta_crop_sq <- dat_mod$prop_to_predict_from_cell_delta_crop^2
  dat_mod$prop_adj_crop_sq <- dat_mod$prop_adj_to_predict_from_crop^2
  # Adding columns for log transformations
  dat_mod$dist_50k_log <- log(dat_mod$dist_50k + .001)
  dat_mod$dist_50k_log_sq <- dat_mod$dist_50k_log^2
  # dat_mod$ISO_numeric <- as.character(dat_mod$ISO_numeric)
  
  # Getting multinomial model
  outcomeName <- "has_changed_crop" # outcome factor variable
  # Attaching data set

  # Getting multinomial regression model
  objModel <- multinom(has_changed_crop ~ dist_50k_log +
                         dist_50k_log_sq +
                         ag_suitability +
                         PA_binary +
                         prop_adj_to_predict_from_crop +
                         prop_adj_crop_sq +
                         prop_to_predict_from_crop +
                         prop_crop_sq +
                         prop_to_predict_from_pasture +
                         prop_pasture_sq +
                         prop_to_predict_from_cell_delta_crop +
                         prop_cell_delta_crop_sq +
                         factor(ISO_numeric, ordered = FALSE),
                       dat = dat_mod,
                       model = FALSE,
                       maxit = 250)
  
  coef.multinom <- coef(objModel)
  
  # 
  # Processing: Creating data frames that only contains cells which have increased or decreasd in extent
  dat_region_increase <- dat_mod[dat_mod$prop_change_cell_crop >= cutoff,]
  dat_region_decrease <- dat_mod[dat_mod$prop_change_cell_crop <= -cutoff,]
  
  # removing data
  rm(dat_mod)
  
  # Dropping rows with values <0 for increaes and > 0 for decrease
  dat_region_increase <- dat_region_increase[dat_region_increase$prop_change_cell_crop_2007.2012 >= 0,]
  dat_region_decrease <- dat_region_decrease[dat_region_decrease$prop_change_cell_crop_2007.2012 <= 0,]
  
  # Getting model for extent increase
  objModel_increase <- glm(prop_change_cell_crop_2007.2012 ~ 
                            dist_50k_log +
                            dist_50k_log_sq +
                            ag_suitability +
                            PA_binary +
                            prop_adj_to_predict_from_crop +
                            prop_adj_crop_sq +
                            prop_to_predict_from_crop +
                            prop_crop_sq +
                            prop_to_predict_from_pasture +
                            prop_pasture_sq +
                            prop_to_predict_from_cell_delta_crop +
                            prop_cell_delta_crop_sq +
                            factor(ISO_numeric, ordered = FALSE),
                            family = Gamma(link = 'log'),
                            dat = dat_region_increase,
                          model = FALSE,
                          control = list(maxit = 250))
  
  coef.increase <- as.data.frame(coef(objModel_increase))
  
  # Getting absolute of decrease in crop extent
  dat_region_decrease$prop_change_cell_crop_2007.2012 <- abs(dat_region_decrease$prop_change_cell_crop_2007.2012)
  objModel_decrease <- glm(prop_change_cell_crop_2007.2012 ~ 
                             dist_50k_log +
                             dist_50k_log_sq +
                             ag_suitability +
                             PA_binary +
                             prop_adj_to_predict_from_crop +
                             prop_adj_crop_sq +
                             prop_to_predict_from_crop +
                             prop_crop_sq +
                             prop_to_predict_from_pasture +
                             prop_pasture_sq +
                             prop_to_predict_from_cell_delta_crop +
                             prop_cell_delta_crop_sq +
                             factor(ISO_numeric, ordered = FALSE),
                           family = Gamma(link = 'log'),
                           dat = dat_region_decrease,
                           model = FALSE,
                           control = list(maxit = 250))
  
  coef.decrease <- as.data.frame(coef(objModel_decrease))

  save(objModel_decrease, objModel_increase, objModel,
       file = paste("Outputs/Model_Coefficients/",
                    region_list[k],
                    "_Gamma_Link_Log_Multinom_Crop_Model.rda",
                    sep = ""))
  
  rm(objModel_decrease, objModel_increase)

  save(coef.increase, coef.decrease, coef.multinom,
       file = paste("Outputs/Model_Coefficients/",
                    region_list[k],
                    "_Gamma_Link_Log_Multinom_Crop_Coefs.rda",
                    sep = ""))

  
}

Sys.time() - t1

quit(save = 'no')