#!/usr/bin/env Rscript

# Read me -----
# Overlaps agricultural projections (from RasterMosaic_Crop.R and 
# RasterMosaic_Pasture.R) with maps of suitable habitat
# This produces a dataframe with every species and the overlap with agricultural lands

# Packages and clean up -----
library(raster)
library(plyr)
library(dplyr)
library(purrr)
library(gdalUtils)
library(rgdal)
library(tidyr)
library(data.table)
library(doMC)

rm(list=ls())

# How many cores?
n_cores <- 20
registerDoMC(cores = n_cores)

# How much memory can raster use?
rasterOptions(maxmemory = 1e+11)

# Directories info ---- 
# This is set up so that the ag rasters and the ESH rasters are stored on local 
#     scratch space ("/scratch/williams/LandExpansionBiodiversity"). 
# This is because they are mahoosive

# The exact directory for the ag rasters will vary depending on the scenario, that for 
#     the ESH files will vary depending on the taxa

# Which taxa and scenario? ----
# Obviously adjust this to whatever you have
taxa_list <- c("Amphibians", "Birds", "Mammals")
scenario_list <- c("BAU", "Yields", "Diets", "Half_Waste", "Trade", "Combined")

for(i in 1:length(taxa_list)){
    taxon <- taxa_list[i]
  for(j in 1:length(scenario_list)){
    scenario <- scenario_list[j]
    # Do we need to do this?
    if(!file.exists(paste("Outputs/ESHTables/", scenario, "/ESHLoss_", 
                          scenario, taxon, ".csv", sep = ""))){
      print(paste("starting", taxon, scenario))
      # Get directories -----
      # Directory containing the ESH rasters
      # NOTE: Please contact us if you do not have these—-we are not allowed to share them freely
      esh_folder = paste("Data/ESHFiles/",
                         taxon, 
                         sep = "")
      # Directory containing agricultural rasters—-these are not included in the dryad repo, but we will happily share them
      agri_folder = paste("Outputs/WorldAgProjections/",
                          scenario, 
                          sep = "")
      
      # Getting list of rasters and creating dataframe ----
      ag_raster_list <- list.files(path = agri_folder,
                                   recursive = TRUE,
                                   full.names = TRUE)
      # Remove any "change" rasters there
      if(length(ag_raster_list[grep("Delta", ag_raster_list)]) > 1){
        ag_raster_list <- ag_raster_list[-grep("Delta", ag_raster_list)]
      }
      if(length(ag_raster_list[grep("sd", ag_raster_list)]) > 1){
        ag_raster_list <- ag_raster_list[-grep("sd", ag_raster_list)]
      }
      
      # Getting 2010 raster file name
      crop_2010 <- "Data/ProportionCrop_2010.tif"
      pasture_2010 <- "Data/ProportionPasture_2010.tif"
      # And appending to beginning of list
      ag_raster_list <- c(crop_2010, pasture_2010,
                          ag_raster_list)
      
      # Importing data frame with tifs
      species_rasters <- list.files(esh_folder,
                                    pattern = "\\.tif$",
                                    recursive = TRUE,
                                    full.names = TRUE)
      # Creating a data frame of rasters etc
      # This will have to be tweaked depending on file names
      species_frame <- data.frame(species_raster = species_rasters,
                                  stringsAsFactors = FALSE) %>%
        mutate(species = gsub(paste(esh_folder,"/", sep = ""), "", species_raster),
               species = gsub(paste(c("_HabElev.*$", "_P1.*$", "_P2.*$", "_P3.*$", "_P4.*$", "_P5.*$"),
                                    collapse = "|"), 
                              "",
                              species),
               subspecies = gsub(paste(esh_folder,"/", sep = ""), "", species_raster),
               subspecies = gsub("_HabElev.*$", "", subspecies)) %>%
        uncount(length(unique(ag_raster_list))) %>%
        mutate(ag_raster = rep(unique(ag_raster_list), length(species_rasters)),
               year = gsub(paste(c("^.*/Pasture_", "^.*/Crop_", "_mean.tif"), collapse = "|"),
                           "", ag_raster),
               year = ifelse(grepl("2010", ag_raster),
                             yes = 2010, 
                             no = gsub("^.*_", "", year)),
               year = ifelse(grepl("NoHabitatRecovery", ag_raster),
                             yes = 2051, # Setting the year to 2051 for no habitat recovery
                             no = year),
               year = as.numeric(gsub("\\.tif", "", year)),
               ag_type = ifelse(grepl("Crop", ag_raster),
                                yes = "crop", 
                                no = "pasture")) %>%
        arrange(ag_raster)
      
      # Reduce to just 2010 and 2050 for gridded pasture (and the 2051 for no regrowth)
      species_frame <- species_frame %>% 
        filter(year %in% c(2010, 2050, 2051))
      
      # Functions to use ----
      by_year_ag_fun <- function(dataframe){
        ag_raster <- raster(dataframe$ag_raster[1])
        year_df <- dataframe %>%
          group_by(species, subspecies, species_raster) %>%
          do(by_species_fun(df = ., ag_raster = ag_raster))
        removeTmpFiles(h = .025) # Use this if it's super slow?
        return(year_df)
      }
      
      by_species_fun <- function(df, ag_raster){
        esh_raster <- raster(df$species_raster[1])
        tmp_crop_raster <- ag_raster * esh_raster
        sp_df <- data.frame(
          esh_loss = cellStats(tmp_crop_raster, "sum"),
          esh_area_orig = ncell(tmp_crop_raster) - cellStats(tmp_crop_raster, "countNA"))
        return(sp_df)
      }
      
      wrap_fun <- function(df){
        out <- df %>%
          group_by(ag_raster, year, ag_type) %>%
          do(by_year_ag_fun(dataframe = .))
        return(out)
      }
      
      # Split dataframe -----
      splits <- c()
      for(i in 1:(n_cores - 1)){
        if(i == 1){
          splits[i] <- ceiling(nrow(species_frame) / n_cores)
          } else {
            splits[i] <- ceiling(nrow(species_frame) / n_cores) + splits[i - 1]
          }
        }
      split_df <- split(x = species_frame, f = cumsum(1:nrow(species_frame) %in% splits))
      names(split_df) <- as.character(1 + as.numeric(names(split_df)))
      
      # Run across multiple cores ----
      t1 <- Sys.time()
      df_list <- foreach(i = 1:n_cores) %dopar% wrap_fun(df = split_df[[i]])
      names(df_list) <- c(1:length(df_list))
      Sys.time() - t1 
      
      # Recombine dataframe ----
      esh_df <- data.frame(rbindlist(df_list))
      
      # Save -----
      write.csv(esh_df,
                file = paste("Outputs/ESHTables/", scenario, "/ESHLoss_", 
                             scenario, taxon, ".csv", sep = ""),
                row.names = FALSE)
    } else {
      print(paste(taxon, scenario, "already completed"))
    }
  }
}

