# Read me ----
# Calculates the loss of suitable habitat for each species based on 
# a) the overlap with agricultural land from OverlayingAOHWithLandUseChange.R 
# b) habitat suitability

# Loading packages
library(raster)
library(plyr)
library(dplyr)
library(tidyr)
library(purrr)
options(tibble.width = Inf)
library(ggplot2)
library(data.table)

# Clean up ----
rm(list=ls())

# Step 0: Getting cell areas ----
tmp.raster <- raster("Data/ProportionCrop_2010.tif")
area <- (res(tmp.raster)[1] * res(tmp.raster)[2])/1e6
rm(tmp.raster)

# Step 1: Importing, rearranging ESH results ----
# Getting list of ESH tables
esh_files_names <- list.files(path = "Outputs/ESHTables",
                              pattern = ".csv",
                              recursive = TRUE,
                              full.names = TRUE)

# Import files and combine them into a single, massive dataframe
esh_files <- map(esh_files_names, fread)
names(esh_files) <- esh_files_names

for(i in 1:length(esh_files)){
  esh_files[[i]][["id"]] <- names(esh_files)[i]
}
esh_df <- data.frame(rbindlist(esh_files))

# Inexplicably didn't cut the SD rasters out...
esh_df <- esh_df %>% 
  filter(!grepl("sd", ag_raster))

# Cut the unnecessary columns, check that species don't have .tif, add taxa and scenario
esh_df <- esh_df %>%
  mutate(species = gsub("\\.tif$", "", species),
         id = gsub("(^.*ESHLoss_)(.*)(\\.csv)", "\\2", id),
         id = gsub("griddedlivestock_harmonised", "", id),
         taxon = gsub("^.*_", "", id),
         scenario = gsub("_.*$", "", id)) %>%
  select(taxon, species, subspecies, 
         id, scenario, year, ag_type, 
         esh_loss, esh_area_orig)

# Filter to just 2010 and 2050 ----
# Keep 2051 as well, which is "no-recovery-from-cropland"
esh_df <- esh_df %>%
  filter(year %in% c(2010, 2050, 2051))

# Double up pasture 2050 to merge with no-recovery-from-cropland
esh_df <- esh_df %>%
  filter(year == 2050,
         ag_type == "pasture") %>%
  mutate(year = 2051) %>%
  bind_rows(esh_df, .) %>%
  arrange(taxon, species, scenario, year, ag_type)

# Quickly check that each year / scenario combo has two entries

esh_df %>%
  group_by(scenario) %>%
  do(data.frame(table(.$year, .$ag_type))) %>%
  filter(Freq != 20343) 

# Spread df so each year / scenario combo has only one row per species
esh_df <- esh_df %>%
  gather(variable, value, -(taxon:ag_type)) %>%
  unite(tmp, ag_type, variable) %>%
  spread(tmp, value) %>%
  # We only need one "original" area
  select(taxon:year, 
         esh_area_orig = crop_esh_area_orig, 
         crop_esh_loss, pasture_esh_loss)

# We want 2010 values in separate columns as these are adjustments, not losses (i.e. to 
#     harmonise our changes with our starting point data, which may differ from IUCN data)
# I'm also spreading out habitat regrowth and non-regrowth into separate columns
esh_df <- esh_df %>%
  filter(year == 2010) %>%
  select(taxon:scenario,
         esh_loss_crop_2010 = crop_esh_loss,
         esh_loss_pasture_2010 = pasture_esh_loss) %>%
  left_join(esh_df, .) %>%
  select(taxon:esh_area_orig,
         esh_loss_crop_2010, esh_loss_pasture_2010,
         esh_loss_crop = crop_esh_loss, 
         esh_loss_pasture = pasture_esh_loss)

# Step 2: Adding in habitat preferences----
# Load and rearrange data 
bird_preferences <- read.csv("Data/HabitatSuitabilityData/BirdHabitats2017.csv",
                             stringsAsFactors = FALSE)
mammal_preferences <- read.csv("Data/HabitatSuitabilityData/MammalHabitats2017.csv",
                               stringsAsFactors = FALSE)
amphibian_preferences <- read.csv("Data/HabitatSuitabilityData/AmphibianHabitats2018.csv",
                                  stringsAsFactors = FALSE)

# Get standardised names (won't be necessary in final, cleaned scripts)
mammal_preferences = rename(mammal_preferences,
                            SIS.ID = assessment.id,
                            WBDB.ID = taxon.id,
                            Family = Family,
                            Scientific.name = Species,
                            Habitats.Classification.Scheme.Level.2 = Habitat.description,
                            X2017.IUCN.Red.List.Category = Category)

amphibian_preferences <- amphibian_preferences %>%
  rename(WBDB.ID = Species_ID,
         Scientific.name = species,
         Habitats.Classification.Scheme.Level.2 = Habitat,
         Suitability = Suitability) %>%
  # And changing format of habitat classification type 2
  mutate(Habitats.Classification.Scheme.Level.2 = sub("^\\S+\\s+", '', 
                                                       Habitats.Classification.Scheme.Level.2)) %>%
  filter(!is.na(WBDB.ID))

# Bind them all together
habitat_preferences <- amphibian_preferences %>%
  mutate(taxon = "Amphibians") %>%
  bind_rows(., bird_preferences %>%
              mutate(taxon = "Birds")) %>%
  bind_rows(., mammal_preferences %>%
              mutate(taxon = "Mammals")) %>%
  select(SIS.ID, WBDB.ID, taxon, Family,
         species = Scientific.name, 
         X2017.IUCN.Red.List.Category, 
         Habitats.Classification.Scheme.Level.2, 
         Major.importance., Suitability) %>%
  filter(!is.na(species)) %>%
  mutate(species = sub(" ", "_", species),
         # Are species specialists or generalists?
         suitable = ifelse(Suitability == "Suitable",
                           yes = 1, no = 0),
         marginal = ifelse(Suitability == "Marginal",
                           yes = 0.5, no = 0))

# Estimating species' tolerance for agricultural lands----
cropland_habitats <- c("Artificial/Terrestrial - Arable Land",
                       "Artificial/Terrestrial - Plantations")
pastureland_habitats <- c("Artificial/Terrestrial - Pastureland")

ag_suitability <- habitat_preferences %>%
  mutate(is_ag = ifelse(Habitats.Classification.Scheme.Level.2 %in% c(cropland_habitats),
                                 yes = "cropland_suitability",
                                 no = ifelse(Habitats.Classification.Scheme.Level.2 %in% c(pastureland_habitats),
                                             yes = "pastureland_suitability",
                                             no = "not_ag")),
         is_ag = factor(is_ag),
         species = factor(species)) %>%
  select(-Suitability, -Major.importance.) %>%
  gather(tmp, suitability_value, 
         -c(SIS.ID:Habitats.Classification.Scheme.Level.2, is_ag)) %>%
  select(-tmp) %>%
  complete(species, is_ag, fill = list(suitability_value = 0)) %>%
  filter(is_ag != "not_ag") %>%
  group_by(species, is_ag) %>%
  summarise(highest_suitability = max(suitability_value, na.rm = TRUE)) %>%
  spread(is_ag, highest_suitability) %>%
  ungroup()

# Bind back to ESH data --- 
esh_df <- esh_df %>%
  left_join(., ag_suitability)

# Analyses: AOH loss -----
esh_df <- esh_df %>%
  mutate(
    AOH_Lost_2010 = esh_loss_crop_2010 * (1 - cropland_suitability) + 
           esh_loss_pasture_2010 * (1 - pastureland_suitability),
    # AOH loss in each year
    AOH_Lost = esh_loss_crop * (1 - cropland_suitability) + 
           esh_loss_pasture * (1 - pastureland_suitability),
    # Calculate this as a percentage
    # How much remains?
    AOH_Remaining_2010 = esh_area_orig - AOH_Lost_2010,
    AOH_Remaining = esh_area_orig - AOH_Lost,
    # Calculate loss as a percentage
    Percent_Change_in_Remaining_AOH = (AOH_Remaining / AOH_Remaining_2010) * 100 - 100,
    Percent_Remaining_AOH_Lost = -Percent_Change_in_Remaining_AOH
    )

# Cut species with no habitat originally (i.e. bad ESH maps) ----
esh_df <- esh_df %>%
  filter(esh_area_orig > 0)

# Save file ----
write.csv(esh_df,
          "Data/AOHLossTable.csv",
          row.names = FALSE)
